//Copyright 2022 Hans Moeller. Licenced under the GNU GPL Version 3.
//
//
// This Dummy Load controller project is distributed,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

#include "cli.h"

//Top level functions
void cli_init(){
	Serial.begin(9600);
	Serial.setTimeout(8000);
    Serial.println("TRX-Mic-Controller command line interface (CLI).");
    Serial.println("Type \"help\" to see a list of commands.");
}


void run_cli(){
    Serial.print("> ");

    read_line();
   if(!error_flag){
        parse_line();
    }
    if(!error_flag){
        execute();
    }

    memset(line, 0, LINE_BUF_SIZE);
    memset(args, 0, sizeof(args[0][0]) * MAX_NUM_ARGS * ARG_BUF_SIZE);

    error_flag = false;
}

//Core functions
void read_line(){

    String line_string;

    while(!Serial.available());

    if(Serial.available()){
        line_string = Serial.readStringUntil(13);
        if(line_string.length() < LINE_BUF_SIZE){
          line_string.toCharArray(line, LINE_BUF_SIZE);
          Serial.println("");
          Serial.print(">>");
          Serial.println(line_string);
        }
        else{
          Serial.println("Input string too long.");
          error_flag = true;
        }
    }
}

void parse_line(){
    char *argument;
    int counter = 0;

    argument = strtok(line, ",");

    while((argument != NULL)){
        if(counter < MAX_NUM_ARGS){
            if(strlen(argument) < ARG_BUF_SIZE){
                strcpy(args[counter],argument);
                argument = strtok(NULL, ",");
                counter++;
            }
            else{
                Serial.println("Input string too long.");
                error_flag = true;
                break;
            }
        }
        else{
            break;
        }
    }
}

int execute(){

    for(int i=0; i<num_commands; i++){
        if(strcmp(args[0], commands_str[i]) == 0){
            return(*commands_func[i])();
        }
    }

    Serial.println("Invalid command. Type \"help\" for more.");
    return 0;
}

/*****************************************************************/
/*  CLI Help  													 */
/*****************************************************************/
int cmd_help(){
    if(args[1] == NULL){
        help_help();
    }
    else if(strcmp(args[1], commands_str[0]) == 0){
        help_help();
    }
    else if(strcmp(args[1], commands_str[1]) == 0){
        help_trx();
    }
    else if(strcmp(args[1], commands_str[2]) == 0){
        help_exit();
    }
    else{
        help_help();
    }
}

void help_help(){
    Serial.println("The following commands are available:");

    for(int i=0; i<num_commands; i++){
        Serial.print("  ");
        Serial.println(commands_str[i]);
    }
    Serial.println("");
    Serial.println("You can for instance type \"help,trx\" for more info on the TRX command.");
}

void help_trx(){
    Serial.print("Usage: trx,TRXnumber,TRXname [max 15 character] ");

}

void help_exit(){
    Serial.println("This will stop the CLI");
}

/*****************************************************************/
/*  CLI Control Functions										 */
/*****************************************************************/

int cmd_trx(){

	unsigned int trx = atoi(args[1]);

	if(trx >= 1 && trx <=8){				//limit for valid TRX no range
		trx--;								//Reduce for 0..7 range
		memcpy(trxName[trx],args[2],15);	//Update Nname

		for (int i = 0; i<15; i++){			//Write to EEPROM
			EEPROM.write((EEPROM_TRX_NAME + (trx*15)+i),args[2][i]);
		}

	}
	else Serial.println("Invalid TRX no.");

}

int cmd_exit(){
    Serial.println("Exiting CLI.");
    statusButtons = 1;					//simulating a pressed button to exit the CLI


}
