//Copyright 2022 Hans Moeller. Licenced under the GNU GPL Version 3.
//
//
// This Dummy Load controller project is distributed,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.

#include "trx_mic_ctl.h"

void setup() {


// IO Settings
	digitalWrite(PIN_OE, HIGH);		//Keep Tri-State active at this moment. Write output berfore pinmode to avoid the pin going to LOW


    pinMode(PIN_CLK, OUTPUT);
    pinMode(PIN_SDA, OUTPUT);
    pinMode(PIN_SCL, OUTPUT);
    pinMode(PIN_RCK, OUTPUT);
    pinMode(PIN_OE, OUTPUT);

    digitalWrite(PIN_SCL, HIGH);	//Release from Reset

    pinMode(BTN_R1,INPUT_PULLUP);
    pinMode(BTN_R2,INPUT_PULLUP);
    pinMode(BTN_S1,INPUT_PULLUP);
    pinMode(BTN_S2,INPUT_PULLUP);
    pinMode(BTN_ENC,INPUT_PULLUP);
    pinMode(PTT1_IN,INPUT_PULLUP);
 //   pinMode(PTT2_IN);

//Initialize LCD module
	Lcd.begin(20, 2);
	Lcd.clear();
	Lcd.setCursor(0, 0);
	Lcd.print("AB5HM TRX-MIC.");
	Lcd.setCursor(0, 1);
	Lcd.print("Controller V2.0");


//Initialize timer1
	noInterrupts();           // disable all interrupts
	TCCR1A = 0;
    TCCR1B = 0;
    TCNT1  = 0;
	OCR1A = 3125;            // compare match register 16MHz/256/50ms
	TCCR1B |= (1 << WGM12);   // CTC mode
    TCCR1B |= (1 << CS12);    // 256 prescaler
    TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt

//Enable interrupts
    attachInterrupt(0, rotate, CHANGE);		//Define interrupt call on rotary pin1
    attachInterrupt(1, rotate, CHANGE);		//Define interrupt call on rotary pin2
    interrupts();            				 // enable all interrupts

    InitExtIO();

    menuMode = 0;

 //EEPROM
    for(int i = EEPROM_TRX_LEVEL; i <= 8; i++){
       	audioLevel[i] = EEPROM.read(i);
        }

     activeTrx1 = EEPROM.read(EEPROM_ACT_TRX_1);
     if (activeTrx1 > 7) activeTrx1 = 7;

     activeTrx2 = EEPROM.read(EEPROM_ACT_TRX_2);
     if (activeTrx2 > 7) activeTrx2 = 7;


     micGain[0] = EEPROM.read(EEPROM_MIC1);
     micGain[1] = EEPROM.read(EEPROM_MIC2);
     upDownMode = EEPROM.read(EEPROM_UD_MODE);
     if (upDownMode > 2) upDownMode = 2;

     digitalPotWrite(0, micGain[0]);		//Load preAmp Gain
     digitalPotWrite(1, micGain[1]);


 //     memcpy(trxName[0],"KENWOOD TS-570S",15);
 //     memcpy(trxName[1],"STANDARD C5608D",15);
 //     memcpy(trxName[2],"MOTO MC-micro 1",15);
//      memcpy(trxName[3],"MOTO MC-micro 2",15);
//      memcpy(trxName[4],"PENUT          ",15);
//      memcpy(trxName[5],"PENUT 1        ",15);
//      memcpy(trxName[6],"PENUT 2        ",15);
//      memcpy(trxName[7],"PENUT 3        ",15);

     //read TRX names from EEPROM
      for (int t = 0; t<8; t++){
		  for (int i = 0; i<15; i++){
			  trxName[t][i] = EEPROM.read(EEPROM_TRX_NAME + (t*15)+i);
		 }
      }

     delay(1000);
  //   activeTrx1 = 1;  //Tranceiver 1 aktivieren
     encoderCounter = activeTrx1;
     statusButtons = 0;
     updateLCD = true;
     timer_Btn = 2;

     SetExtIO(PTT_LED1,HIGH);
     SetExtIO(PTT_LED2,HIGH);

}
/*****************************************************************/
/* Interrupt routines											 */
/*****************************************************************/
ISR(TIMER1_COMPA_vect)    // timer1 compare interrupt service routine  50ms
{
	if (timer_Btn) 		{	timer_Btn--;}
	if (timer_Lcd) 		{	timer_Lcd--;}
	if (timer_AutoSave) {timer_AutoSave--;}



}

// rotate is called anytime the rotary inputs change state.
void rotate() {

	  unsigned char result = rotary.process();
	  if (result == DIR_CW) {
		 encoderCounter++;
 	  }
	  else if (result == DIR_CCW) {
		encoderCounter--;
	  }

}
/*****************************************************************/
/* Software SPI      											 */
/*****************************************************************/
void spi_transfer(byte working) {
// function to actually bit shift the data byte out


	for(int i = 1; i <= 8; i++) { // setup a loop of 8 iterations, one for each bit
		if (working > 127) { // test the most significant bit
			digitalWrite (PIN_SDA,HIGH); // if it is a 1 (ie. B1XXXXXXX), set the master out pin high
		}

		else {
			digitalWrite (PIN_SDA, LOW); // if it is not 1 (ie. B0XXXXXXX), set the master out pin low
		}

	digitalWrite (PIN_CLK,HIGH); // set clock high, the pot IC will read the bit into its register
	working = working << 1;
	digitalWrite(PIN_CLK,LOW); // set clock low, the pot IC will stop reading and prepare for the next iteration (next significant bit

	}


}

void spi_out(int SS, byte cmd_byte, byte data_byte) { // SPI tranfer out function begins here

	byte work = 0;

	SetExtIO (SS, LOW); // set slave select low for a certain chip, defined in the argument in the main loop. selects the chip
	work = cmd_byte; // let the work byte equal the cmd_byte, defined in the argument in the main loop
	spi_transfer(work); // transfer the work byte, which is equal to the cmd_byte, out using spi
	work = data_byte; // let the work byte equal the data for the pot
	spi_transfer(work); // transfer the work byte, which is equal to the data for the pot
	digitalWrite(PIN_OE, HIGH);
	SetExtIO(SS, HIGH); // set slave select high for a certain chip, defined in the argument in the main loop. deselcts the chip
	digitalWrite(PIN_OE, LOW);

}


/*****************************************************************/
/* Digital Potentiometer										 */
/*****************************************************************/
void digitalPotWrite(int Ch,int value){

//	value = 255 - value; // Poti Richtung inkrementieren

//	if (value>=256) cmd_byte = 0x01;		//the 9th bit of the value is located in the command byte
//	else cmd_byte = 0x00;

	delay(1);
	switch(Ch){

	case 0:	//Mic #1
		spi_out(1,cmd_byte,micGainSetp[value]);
		break;

	case 1: //Mic #2
		spi_out(0,cmd_byte,micGainSetp[value]);
		break;

	case 2: //TRX
		spi_out(2,cmd_byte,trxAttSetp[value]);
		break;

	}
delay(1);
}

/*****************************************************************/
/* Read push button status   									 */
/*****************************************************************/
void ReadButtons(void)
{


	pinMode(BTN_S1,OUTPUT);
	digitalWrite(BTN_S1,LOW);

	while (digitalRead(BTN_R1) == false)	statusButtons = BTN_2_PRESSED ;
	while (digitalRead(BTN_R2)== false)		statusButtons = BTN_1_PRESSED ;

	digitalWrite(BTN_S1,HIGH);
	pinMode(BTN_S1,INPUT_PULLUP);

	pinMode(BTN_S2,OUTPUT);
	digitalWrite(BTN_S2,LOW);

	while (digitalRead(BTN_R1)== false)		statusButtons = BTN_3_PRESSED ;
	while (digitalRead(BTN_R2)== false)		statusButtons = BTN_4_PRESSED ;

	digitalWrite(BTN_S2,HIGH);
	pinMode(BTN_S2,INPUT_PULLUP);

	while (digitalRead(BTN_ENC) == false)	statusButtons = BTN_ENC_PRESSED ;

	//Mic Up Down buttons via analog input
	static unsigned int _Uold;
	unsigned int U;

	while (analogRead(PIN_UD) < 1000){
		U = (analogRead(PIN_UD));						//Read ADC
	//	U = ((_Uold * ADC_FILTER ) + U) / (ADC_FILTER + 1); //Filtern
	//	_Uold = U;
		delay(2);
		if (U < 850 && U > 700) statusButtons = BTN_UP_PRESSED ;
		if (U < 450 && U > 300) statusButtons = BTN_DOWN_PRESSED ;
	}


}



/*****************************************************************/
/* Operation Control		   									 */
/*****************************************************************/
void Control(void){

	static bool startTX = false;
	static bool startVOX = false;
	static unsigned char oldVoxMode = 0;
	static unsigned char trxNo = 0;
	bool ptt2;
	bool ptt1 = digitalRead(PTT1_IN);	//Read PTT1

	if (ptt1 == false) lastPtt = false;	//PTT1 was used

	if (analogRead(PTT2_IN) <= 500){	//Read PTT2, A6 is an analog only pin
		ptt2 = false;
		lastPtt = true;					//PTT2 was used
	}
	else ptt2 = true;


	//VOX Contol
	if (voxMode >0 || startVOX == true){

		startVOX = true;
		switch (voxMode) {

		case 0:		if (oldVoxMode != voxMode){
						oldVoxMode = voxMode;
						SetExtIO(6,LOW);
						SetExtIO(8,LOW);
						SetExtIO(MIC_SEL,LOW);
						SetExtIO(PTT_LED1,HIGH);		//PTT LED
						SetExtIO(PTT_LED2,HIGH);		//PTT LED
						startVOX = false;
					}
					break;


		case 1:		if (oldVoxMode != voxMode){
						oldVoxMode = voxMode;
						SetExtIO(MIC_SEL,HIGH);		//Select mic
						digitalPotWrite(2,audioLevel[0]);		//Set gain for TRX 1
						SetExtIO(6,HIGH);			//Activate TRX 1 MIC
						SetExtIO(8,LOW);			//Disable TRX 2 MIC
						SetExtIO(PTT_LED1,LOW);		//PTT LED
						SetExtIO(PTT_LED2,HIGH);		//PTT LED
					}
					break;

		case 2:		if (oldVoxMode != voxMode){
						oldVoxMode = voxMode;
						SetExtIO(MIC_SEL,LOW);		//Select mic
						digitalPotWrite(2,audioLevel[0]);		//Set Mic gain for TRX 1
						SetExtIO(6,HIGH);			//Activate TRX 1 MIC
						SetExtIO(8,LOW);			//Disable TRX 2 MIC
						SetExtIO(PTT_LED1,LOW);		//PTT LED
						SetExtIO(PTT_LED2,HIGH);		//PTT LED
					}
					break;

		case 3:		if (oldVoxMode != voxMode){
						oldVoxMode = voxMode;
						SetExtIO(MIC_SEL,HIGH);		//Select mic
						digitalPotWrite(2,audioLevel[1]);		//Set Mic gain for TRX 2
						SetExtIO(6,LOW);			//Disable TRX 1 MIC
						SetExtIO(8,HIGH);			//Activate TRX 2 MIC
						SetExtIO(PTT_LED1,HIGH);		//PTT LED
						SetExtIO(PTT_LED2,LOW);		//PTT LED
					}
					break;


		case 4:		if (oldVoxMode != voxMode){
						oldVoxMode = voxMode;
						SetExtIO(MIC_SEL,LOW);		//Select mic
						digitalPotWrite(2,audioLevel[0]);		//Set Mic gain for TRX 2
						SetExtIO(6,LOW);			//Disable TRX 1 MIC
						SetExtIO(8,HIGH);			//Activate TRX 2 MIC
						SetExtIO(PTT_LED1,HIGH);	//PTT LED
						SetExtIO(PTT_LED2,LOW);		//PTT LED
					}
					break;

		}


		return;
	}

	//Start TX with PTT
	if ((ptt1 == false || ptt2 == false) && startTX == false){
		startTX = true;


		if (ptt1 == false){
			trxNo = activeTrx1;			//Set TRX
			SetExtIO(MIC_SEL,HIGH);		//Select mic
			digitalPotWrite(2,audioLevel[trxNo]);	//Set TRX gain for selected TRX
			SetExtIO(PTT_LED1,LOW);		//PTT LED
		}

		if (ptt2 == false){
			trxNo = activeTrx2;			//Set TRX
			SetExtIO(MIC_SEL,LOW);		//Select mic
			digitalPotWrite(2,audioLevel[trxNo]);	//Set TRX gain for selected TRX
			SetExtIO(PTT_LED2,LOW);		//PTT LED
		}

		if (trxNo == 0){				//TRX1 and TRX2 have two separate relais to support VOX mode
			SetExtIO(6,HIGH);			//Set Mic relay
			SetExtIO(7,HIGH);			//Set PTT relay
		}
		else if (trxNo == 1){
			SetExtIO(8,HIGH);			//Set Mic relay
			SetExtIO(9,HIGH);			//Set PTT relay
		}
		else 	SetExtIO( (8 + trxNo),HIGH);	//Set TRX 3..8

	}//Start TX

	//Stop TX with PTT
	if (ptt1 == true && ptt2 == true && startTX == true){
			startTX = false;

			if (trxNo == 0){
				SetExtIO(6,LOW);	//Reset Mic relay
				SetExtIO(7,LOW);	//Reset PTT relay
			}
			else if (trxNo == 1){
				SetExtIO(8,LOW);	//Reset Mic relay
				SetExtIO(9,LOW);	//Reset PTT relay
			}

			else SetExtIO( (8 + trxNo),LOW);  //Reset TRX 3..8

			SetExtIO(PTT_LED1,HIGH);	//Reset PTT LEDs
			SetExtIO(PTT_LED2,HIGH);
	}//Stop TX


}


/*****************************************************************/
/*Main Loop					  									 */
/*****************************************************************/
void loop() {

	static int oldEncCounter;

	if (timer_Btn == 0){
		ReadButtons();
		timer_Btn = 2;
	}

	if (encoderCounter != oldEncCounter){
		updateLCD = true;
		oldEncCounter = encoderCounter;
	}

	//Store selected TRX in EEPROM if changed
	if (timer_AutoSave == 0){
		if(EEPROM.read(EEPROM_ACT_TRX_1) != activeTrx1){ EEPROM.write(EEPROM_ACT_TRX_1,activeTrx1);}
		if(EEPROM.read(EEPROM_ACT_TRX_2) != activeTrx2){ EEPROM.write(EEPROM_ACT_TRX_2,activeTrx2);}
		timer_AutoSave = 1200;  // Timer ca 1 min.
	}



	if ((statusButtons != 0) || (updateLCD == true)) {
		updateLCD = false;
		MenuControl();
	}

	Control();


}
